;; -*- Mode: Lisp; Syntax: ANSI-Common-Lisp -*-

(in-package :iolib-posix)

(defun fd-open-p (fd)
  (with-foreign-object (stat 'et:stat)
    (handler-case
        (progn (et:stat fd stat) t)
      (et:unix-error-badf (err)
        (declare (ignore err))
        nil))))

(defun gettime ()
  (multiple-value-bind (sec nsec)
      (clock-get-time clock-monotonic)
    (+ sec (/ nsec 1d9))))

(defmacro repeat-upon-condition ((&rest conditions) &body body)
  (let (($block$ (gensym "BLOCK-")))
    `(block ,$block$
       (loop
          (handler-case
              (return-from ,$block$ (progn ,@body))
            ,@(mapcar #'(lambda (c) `(,c (err) (declare (ignore err))))
                      conditions))))))

(defmacro repeat-upon-eintr (&body body)
  `(repeat-upon-condition (et:unix-error-intr) ,@body))

(defmacro repeat-upon-condition-decreasing-timeout
    (((&rest conditions) timeout-var timeout) &body body)
  (let (($block$ (gensym "BLOCK-"))
        ($deadline$ (gensym "DEADLINE-"))
        ($temp-timeout$ (gensym "TEMP-TIMEOUT-")))
    `(block ,$block$
       (let* ((,timeout-var ,timeout)
              (,$deadline$ (when ,timeout-var
                             (+ ,timeout-var (gettime)))))
         (flet ((recalculate-timeout ()
                  (when ,$deadline$
                    (let ((,$temp-timeout$ (- ,$deadline$ (gettime))))
                      (setf ,timeout-var
                            (if (plusp ,$temp-timeout$)
                                ,$temp-timeout$
                                0))))))
           (loop
              (handler-case
                  (return-from ,$block$ (progn ,@body))
                ,@(mapcar #'(lambda (c) `(,c (err)
                                             (declare (ignore err))
                                             (recalculate-timeout)))
                          conditions))))))))

(defmacro repeat-decreasing-timeout ((timeout-var timeout &optional block-var)
                                     &body body)
  (let (($block$ (gensym "BLOCK-"))
        ($deadline$ (gensym "DEADLINE-"))
        ($temp-timeout$ (gensym "TEMP-TIMEOUT-")))
    `(block ,$block$
       (let* ,(remove-if #'null
                `(,(when block-var `(,block-var ,$block$))
                   (,timeout-var ,timeout)
                   (,$deadline$ (when ,timeout-var
                                  (+ ,timeout-var (gettime))))))
         (loop
            ,@body
            (when ,$deadline$
              (let ((,$temp-timeout$ (- ,$deadline$ (gettime))))
                (setf ,timeout-var
                      (if (plusp ,$temp-timeout$)
                          ,$temp-timeout$
                          0)))))))))
